package lcsb.mapviewer.api.projects;

import java.io.Serializable;
import java.util.List;

import org.apache.log4j.Logger;

import lcsb.mapviewer.services.view.AnnotationView;
import lcsb.mapviewer.services.view.OverviewImageView;

public class ProjectMetaData implements Serializable {

	private static Logger						logger					 = Logger.getLogger(ProjectMetaData.class);

	/**
	 * 
	 */
	private static final long				serialVersionUID = 1L;

	/**
	 * Version of the project.
	 */
	private String									version;

	private AnnotationView					disease;

	private AnnotationView					organism;

	private Integer									idObject;

	/**
	 * Name of the project.
	 */
	private String									name;

	/**
	 * {@link lcsb.mapviewer.model.Project#projectId Project identifier}.
	 */
	private String									projectId;

	/**
	 * Description of the model.
	 */
	private String									description;

	private ModelMetaData						map;

	private Integer									publicationCount;

	/**
	 * List of overview images attached to this model.
	 */
	private List<OverviewImageView>	overviewImageViews;

	/**
	 * Top level overview image.
	 */
	private OverviewImageView				topOverviewImage;

	/**
	 * Default constructor. Should be used only for deserialization.
	 */
	protected ProjectMetaData() {
	}

	/**
	 * @return the version
	 * @see #version
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * @param version
	 *          the version to set
	 * @see #version
	 */
	public void setVersion(String version) {
		this.version = version;
	}

	/**
	 * @return the description
	 * @see #description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description
	 *          the description to set
	 * @see #description
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * @return the name
	 * @see #name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *          the name to set
	 * @see #name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the overviewImageViews
	 * @see #overviewImageViews
	 */
	public List<OverviewImageView> getOverviewImageViews() {
		return overviewImageViews;
	}

	/**
	 * @param overviewImageViews
	 *          the overviewImageViews to set
	 * @see #overviewImageViews
	 */
	public void setOverviewImageViews(List<OverviewImageView> overviewImageViews) {
		this.overviewImageViews = overviewImageViews;
	}

	/**
	 * @return the topOverviewImage
	 * @see #topOverviewImage
	 */
	public OverviewImageView getTopOverviewImage() {
		return topOverviewImage;
	}

	/**
	 * @param topOverviewImage
	 *          the topOverviewImage to set
	 * @see #topOverviewImage
	 */
	public void setTopOverviewImage(OverviewImageView topOverviewImage) {
		this.topOverviewImage = topOverviewImage;
	}

	/**
	 * @return the projectId
	 * @see #projectId
	 */
	public String getProjectId() {
		return projectId;
	}

	/**
	 * @param projectId
	 *          the projectName to set
	 * @see #projectId
	 */
	public void setProjectId(String projectId) {
		this.projectId = projectId;
	}

	/**
	 * @return the map
	 * @see #map
	 */
	public ModelMetaData getMap() {
		return map;
	}

	/**
	 * @param map
	 *          the map to set
	 * @see #map
	 */
	public void setMap(ModelMetaData map) {
		this.map = map;
	}

	/**
	 * @return the idObject
	 * @see #idObject
	 */
	public Integer getIdObject() {
		return idObject;
	}

	/**
	 * @param idObject
	 *          the idObject to set
	 * @see #idObject
	 */
	public void setIdObject(Integer idObject) {
		this.idObject = idObject;
	}

	/**
	 * @return the disease
	 * @see #disease
	 */
	public AnnotationView getDisease() {
		return disease;
	}

	/**
	 * @param disease
	 *          the disease to set
	 * @see #disease
	 */
	public void setDisease(AnnotationView disease) {
		this.disease = disease;
	}

	/**
	 * @return the organism
	 * @see #organism
	 */
	public AnnotationView getOrganism() {
		return organism;
	}

	/**
	 * @param organism
	 *          the organism to set
	 * @see #organism
	 */
	public void setOrganism(AnnotationView organism) {
		this.organism = organism;
	}

	/**
	 * @return the publicationCount
	 * @see #publicationCount
	 */
	public Integer getPublicationCount() {
		return publicationCount;
	}

	/**
	 * @param publicationCount
	 *          the publicationCount to set
	 * @see #publicationCount
	 */
	public void setPublicationCount(Integer publicationCount) {
		this.publicationCount = publicationCount;
	}

}
